@@ -22,3 +22,15 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include <crl/common/crl_common_on_main_guarded.h>
 #include <crl/common/crl_common_guards.h>
 #include <crl/qt/crl_qt_guards.h>
+
+#ifdef CRL_ENABLE_RPL_INTEGRATION
+
+#include <rpl/producer.h>
+
+namespace crl {
+
+rpl::producer<> on_main_update_requests();
+
+} // namespace crl
+
+#endif // CRL_ENABLE_RPL_INTEGRATION
