@@ -9,12 +9,13 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 
 #ifdef CRL_USE_WINAPI_TIME
 
-#include <windows.h>
+#include <crl/winapi/crl_winapi_windows_h.h>
 
 namespace crl::details {
 namespace {
 
 double Frequency/* = 0.*/;
+double ProfileFrequency/* = 0.*/;
 
 } // namespace
 
@@ -22,6 +23,7 @@ void init() {
 	LARGE_INTEGER value;
 	QueryPerformanceFrequency(&value);
 	Frequency = 1000. / double(value.QuadPart);
+	ProfileFrequency = 1000000. / double(value.QuadPart);
 }
 
 inner_time_type current_value() {
@@ -30,8 +32,18 @@ inner_time_type current_value() {
 	return value.QuadPart;
 }
 
-time_type convert(inner_time_type value) {
-	return time_type(value * Frequency);
+time convert(inner_time_type value) {
+	return time(value * Frequency);
+}
+
+inner_profile_type current_profile_value() {
+	LARGE_INTEGER value;
+	QueryPerformanceCounter(&value);
+	return value.QuadPart;
+}
+
+profile_time convert_profile(inner_profile_type value) {
+	return profile_time(value * ProfileFrequency);
 }
 
 } // namespace crl::details
